import { Injectable, UnauthorizedException } from '@nestjs/common';
import { PassportStrategy } from '@nestjs/passport';
import { Strategy as LocalStrategy } from 'passport-local';
import { ExtractJwt, Strategy as JWTStrategy } from 'passport-jwt';
import AuthService from './gitlab';
import { jwtPayload, jwtSecret } from './base';

@Injectable()
export class AuthLocalService extends PassportStrategy(LocalStrategy) {
	constructor(private authService: AuthService) {
		super();
	}

	async validate(username: string, pass: string) {
		const user = await this.authService.validateUser(username, pass);
		if (!user)
			throw new UnauthorizedException('Invalid username or password');
		return user;
	}
}

@Injectable()
export class AuthJwtService extends PassportStrategy(JWTStrategy) {
	constructor(private authService: AuthService) {
		super({
			jwtFromRequest: ExtractJwt.fromExtractors([
				ExtractJwt.fromAuthHeaderAsBearerToken(),
				ExtractJwt.fromBodyField('jwtToken')
			]),
			ignoreExpiration: false,
			passReqToCallback: true,
			secretOrKey: jwtSecret
		});
	}

	async validate(req: Request, payload: jwtPayload) {
		await this.authService.cleanupTokens();
		const token = await this.authService.getToken(payload.jti);
		if (!token)
			throw new UnauthorizedException(
				'Invalid token, please log in again'
			);
		const user = await this.authService.getUser(token.ownerId);
		if (!user || user.id != payload.sub)
			throw new UnauthorizedException(
				'Invalid token, please log in again'
			);
		if (
			user.isGitlabUser &&
			!(await this.authService.verifyGitlabUser(req, user))
		) {
			await this.authService.revokeAll(user);
			throw new UnauthorizedException('Invalid gitlab token');
		}
		// eslint-disable-next-line @typescript-eslint/ban-ts-comment
		// @ts-ignore
		req.token = token;
		return user;
	}
}
